﻿// OakLib sample code for C#
//
// Copyright 2010 - Toradex AG
//
// The following code is provided as is, without any warranty

using System;
using System.Runtime.InteropServices;

namespace Oak_CS
{
    // Step 1: translate the C data structure that are in the Oak.h header file (available in the OakLib package from the Toradex website).
    // In order to work. The memory layout of the structures must match exactly the one of the C structures, hence the use MarshalAsAttribute attributes
    // that help to cope with the specificities of C/C++ (string and array length for example).

    //Declaration of the tChannel structure that is compatible with the C structure defined in the C header file Oak.h
    [StructLayoutAttribute(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct tChannel
    {
        [MarshalAsAttribute(UnmanagedType.I1)] public bool IsSigned;
        public byte BitSize;
        public sbyte UnitExponent;
        public uint Unit;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string UnitStr;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string ChannelName;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string UserChannelName;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string UserChannelName_NV;
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 64, ArraySubType = UnmanagedType.I1)] public byte[] RFU;
    }

    //Declaration of the tOak sensor structure that is compatible with the C structure defined in the C header file Oak.h
    [StructLayoutAttribute(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct tOakSensor
    {
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 256)] public string DevicePath;
        public ushort VID;
        public ushort PID;
        public ushort REV;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string SN;
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 64, ArraySubType = UnmanagedType.I1)] public byte[] RFU;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string DeviceName;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string UserDeviceName;
        [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 24)] public string UserDeviceName_NV;
        public ushort NumChannels;
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst=11)] public tChannel[] Channel;
    }

    //Declaration of the delegate function that will be used as the 'function pointer' for registering callback
    [UnmanagedFunctionPointerAttribute(CallingConvention.StdCall)]
    public delegate void tNotifyFct(ref tOakSensor Sensor, int lParam);

    class Oak
    {
        // Step 2: Declare of the function that are present in the OakLib, With their parameter 'translated' to C# data types
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_Initialize_a();
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_Hotplug_Enable_a();
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_Register_Callback_a(tNotifyFct OnConnect, tNotifyFct OnDisconnect, ushort PID, ushort REV, string SN, string DeviceName, string ChannelName, string UserDeviceName, string UserChannelName, int UserParam);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_DestroyNotifications_a();
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_NotifyConnectedDevices_a();
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern bool Oak_FindSensor_a(ushort PID, ushort REV, string SN, string DeviceName, string ChannelName, string UserDeviceName, string UserChannelName, ref tOakSensor SensorFound);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern bool Oak_Feature_a(string DevicePath, byte[] RptBuffer, bool ExpectResult);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern bool Oak_GetInReport_a(string DevicePath, byte[] RptBuffe, byte InReportLength, ushort Timeout_ms);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern bool Oak_GetCurrentInReport_a(string DevicePath, byte[] RptBuffer, byte InReportLength);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern bool Oak_SendOutReport_a(string DevicePath, byte[] RptBuffer, byte InReportLength, ushort Timeout_ms);
        [DllImport("oaka.dll", CallingConvention = CallingConvention.Cdecl)] public static extern void Oak_Cleanup_a();

        // __________________________ O a k _ P r i n t S e n s o r I n f o ______________________________________
        // Print some information that is provided by the Oak Library during the initialization of the sensor
        // (This routine contains no USB specific code, it just prints data)

        public static void Oak_PrintSensorInfo(ref tOakSensor Sensor)
        {
            Console.WriteLine("===============================================");
            Console.WriteLine("{0} (usr:{1})", Sensor.DeviceName, Sensor.UserDeviceName);
            Console.WriteLine("===============================================\n");
            Console.WriteLine("VID:{0:X4}  PID:%{1:X4}  REV:%{2:X4}  SN:{3}", Sensor.VID, Sensor.PID, Sensor.REV, Sensor.SN);
            Console.WriteLine("\nDevicePath:{0}", Sensor.DevicePath);

            Console.WriteLine("\n  {0:D} Data Channels", Sensor.NumChannels);
            for (int ch = 0; ch < Sensor.NumChannels; ch++)
            {
                Console.WriteLine("\n  ----------");
                Console.WriteLine("  Channel {0:D}", ch);
                Console.WriteLine("  ----------");
                Console.WriteLine("  {0} (usr:{1})", Sensor.Channel[ch].ChannelName, Sensor.Channel[ch].UserChannelName);
                if (Sensor.Channel[ch].IsSigned)
                    Console.WriteLine("  {0:D} Bit signed", Sensor.Channel[ch].BitSize);
                else
                    Console.WriteLine("  {0:D} Bit unsigned", Sensor.Channel[ch].BitSize);
                Console.WriteLine("  Resolution: 1e{0:D} {1}\n", Sensor.Channel[ch].UnitExponent, Sensor.Channel[ch].UnitStr);
            }
        }

        // __________________________ O a k _ P r i n t S e n s o r V a l u e s __________________________________
        // Print measured values
        // This function shows how to use the information provided in the tOakSensor data structure to
        // correctly display the sensor's measured values

        public static void Oak_PrintSensorValues(ref tOakSensor Sensor, byte[] InReport)
        {
            double Val = 0;
            byte RptIdx = 1;

            Console.WriteLine("\n  Current sensor values are:");
            Console.WriteLine("  --------------------------");
            // loop through every data channel
            for (byte i = 0; i < Sensor.NumChannels; i++)
            {
                // evaluate the numeric value of the channel
                // distinguish between 8/16 Bit values and signed/unsigned values
                switch (Sensor.Channel[i].BitSize)
                {
                    case 8:
                        if (Sensor.Channel[i].IsSigned)		// 8bit signed
                            Val = (char)InReport[RptIdx];
                        else
                            Val = (byte)InReport[RptIdx];	// 8bit unsigned
                        RptIdx++;
                        break;

                    case 16:
                        if (Sensor.Channel[i].IsSigned)
                            Val = (int)(InReport[RptIdx] + 256 * InReport[RptIdx + 1]);	// 16bit signed
                        else
                            Val = (ushort)(InReport[RptIdx] + 256 * InReport[RptIdx + 1]);	// 16bit unsigned
                        RptIdx += 2;
                        break;
                }

                // Apply the channel unit exponent
                Val *= Math.Pow(10.0f, Sensor.Channel[i].UnitExponent);

                Console.WriteLine("  {0}: {1} {2}", Sensor.Channel[i].ChannelName, Val, Sensor.Channel[i].UnitStr);
            }
        }

        //Set the device reporting mode to 'After Sample' and the sampling rate to the given value
        //For more details of the signification of the reporting rate and sampling mode, please refer
        //to the Oak Sensors programming guide and your device's datasheet
        public static void SetSamplingParams(ref tOakSensor Sensor, ushort rate)
        {
            //First, set the report mode
            //the format of the feature report is described in the devices' datasheets
            byte[] report = new byte[33];
            report[0] = 0;              //Report #, always 0
            report[1] = 0;              //Set
            report[2] = 0;              //To RAM
            report[3] = 1;
            report[4] = 0;
            report[5] = 0;
            report[6] = 0;              //"After Sampling" mode
            if (!Oak_Feature_a(Sensor.DevicePath, report, false))
            {
                Console.WriteLine("Error: Oak_Feature_a() failed!");
                Environment.Exit(0);
            }

            //Then Set the sample rate
            report[0] = 0;              //Report #, always 0
            report[1] = 0;              //Set
            report[2] = 0;              //To Ram
            report[3] = 2;
            report[4] = 1;
            report[5] = 0;
            report[6] = BitConverter.GetBytes(rate)[0]; //LSB of the rate
            report[7] = BitConverter.GetBytes(rate)[1]; //MSB of the rate
            if (!Oak_Feature_a(Sensor.DevicePath, report, false))
            {
                Console.WriteLine("Error: Oak_Feature_a() failed!");
                Environment.Exit(0);
            }
        }

        // __________________________ O a k _ O n C o n n e c t __________________________________________________
        // This is a Callback function that gets called 
        // - each time a sensor matching specified criteria is plugged in
        // - each time the application calls Oak_NotifyConnectedDevices(), and a sensor matching specified 
        //   criteria is connected to the host
        // Be aware that this function may get called more than once. For example
        // - if more than one sensor matches the specified criteria
        // - if the application calls Oak_NotifyConnectedDevices() more than once.
        public static void Oak_OnConnect(ref tOakSensor Sensor, int Param)
        {
            byte[] InReport = new byte[33];
            Console.WriteLine("\n\n--> Connected: <--");
            SetSamplingParams(ref Sensor, 260);
            Oak_PrintSensorInfo(ref Sensor);

            bool fSuccess = Oak_GetInReport_a(Sensor.DevicePath, InReport, 33, 1933);
            if (!fSuccess)
            {
                Console.WriteLine("/* ERROR: ReadFile failed */");
                Environment.Exit(0);
            }
            else
                Oak_PrintSensorValues(ref Sensor, InReport);
        }

        // __________________________ O a k _ O n D i s c o n n e c t ____________________________________________
        // This is a Callback function that gets called each time a sensor matching specified 
        // criteria is disconnected from the system

        public static void Oak_OnDisconnect(ref tOakSensor Sensor, int Param)
        {
	        Console.WriteLine("\n\n--> Disconnected: <--");
	        Oak_PrintSensorInfo(ref Sensor);
        }

        static void Main(string[] args)
        {
            Oak_Register_Callback_a((tNotifyFct) Oak_OnConnect,	     // function that gets called when a matching sensor appears
                                    (tNotifyFct) Oak_OnDisconnect,   // function that gets called when a matching sensor disappears
                                    0,              // search for ANY PID (product ID)
                                    0,			    // search for ANY REV (product revision)
                                    "",				// search for ANY serial number
                                    "",				// search for ANY device name
                                    "",				// search for ANY channel name
                                    "",				// search for ANY user defined device name
                                    "",				// search for ANY user defined channel name
                                    0);             // This value will be passed to the Callback functions
            // Activate Hotplug functionality
            Oak_Hotplug_Enable_a();

            // Simulate Connecting. This is necessary to be able to detect sensors that are currently already connected.
            // Now "Oak_OnConnect" is called once for every connected matching sensor
            Oak_NotifyConnectedDevices_a();

            // Just wait for the <ENTER> key, before closing the window
            Console.ReadLine();
        }
    }
}
